from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..models.api_status_code import ApiStatusCode
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pagination import Pagination
    from ..models.storage_rule import StorageRule


T = TypeVar("T", bound="ResponseWithStorageRuleList")


@_attrs_define
class ResponseWithStorageRuleList:
    """
    Attributes:
        status_code (Union[Unset, ApiStatusCode]): Members:
            |Value|Description|
            |---|---|
            |1010|OK|
            |1020|Error from database engine|
            |1021|Invalid datasource name|
            |1022|Out of range timestamp|
            |1050|Invalid request|
            |1060|Annotation point was not found|
            |1300|Metric already exist|
            |1301|Metric update failed|
            |1302|Formula metric type mismatch|
            |1303|Changing metric type is not allowed|
            |1305|Name already exists|
            |1310|Default access zone cannot be created/updated/deleted|
            |1350|User already exist in access zone|
            |1360|Invalid file format|
            |1400|Unidentified error|
            |1403|Invalid Authorization|
            |1404|At least one metric is forbidden by permission|
            |1405|At least one of the specified timestamps is in the future|
            |1406|At least one invalid metric name|
            |1500|Internal server error|
            |1501|Warning|
            |1502|Unable to decompress payload|
            |1503|Invalid header content encoding|
        message (Union[Unset, str]):
        data (Union[List['StorageRule'], None, Unset]):
        pagination_info (Union[Unset, Pagination]):
    """

    status_code: Union[Unset, ApiStatusCode] = UNSET
    message: Union[Unset, str] = UNSET
    data: Union[List["StorageRule"], None, Unset] = UNSET
    pagination_info: Union[Unset, "Pagination"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        status_code: Union[Unset, int] = UNSET
        if not isinstance(self.status_code, Unset):
            status_code = self.status_code.value

        message = self.message

        data: Union[List[Dict[str, Any]], None, Unset]
        if isinstance(self.data, Unset):
            data = UNSET
        elif isinstance(self.data, list):
            data = []
            for data_type_0_item_data in self.data:
                data_type_0_item = data_type_0_item_data.to_dict()
                data.append(data_type_0_item)

        else:
            data = self.data

        pagination_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pagination_info, Unset):
            pagination_info = self.pagination_info.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if status_code is not UNSET:
            field_dict["statusCode"] = status_code
        if message is not UNSET:
            field_dict["message"] = message
        if data is not UNSET:
            field_dict["data"] = data
        if pagination_info is not UNSET:
            field_dict["paginationInfo"] = pagination_info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.pagination import Pagination
        from ..models.storage_rule import StorageRule

        d = src_dict.copy()
        _status_code = d.pop("statusCode", UNSET)
        status_code: Union[Unset, ApiStatusCode]
        if isinstance(_status_code, Unset):
            status_code = UNSET
        else:
            status_code = ApiStatusCode(_status_code)

        message = d.pop("message", UNSET)

        def _parse_data(data: object) -> Union[List["StorageRule"], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                data_type_0 = []
                _data_type_0 = data
                for data_type_0_item_data in _data_type_0:
                    data_type_0_item = StorageRule.from_dict(data_type_0_item_data)

                    data_type_0.append(data_type_0_item)

                return data_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["StorageRule"], None, Unset], data)

        data = _parse_data(d.pop("data", UNSET))

        _pagination_info = d.pop("paginationInfo", UNSET)
        pagination_info: Union[Unset, Pagination]
        if isinstance(_pagination_info, Unset):
            pagination_info = UNSET
        else:
            pagination_info = Pagination.from_dict(_pagination_info)

        response_with_storage_rule_list = cls(
            status_code=status_code,
            message=message,
            data=data,
            pagination_info=pagination_info,
        )

        return response_with_storage_rule_list
